@@ -2,6 +2,7 @@
 Moran's I Spatial Autocorrelation Statistics
 
 """
+
 __author__ = (
     "Sergio J. Rey <srey@asu.edu>, "
     "Dani Arribas-Bel <daniel.arribas.bel@gmail.com>, "
@@ -11,8 +12,11 @@ __author__ = (
 from warnings import simplefilter
 
 import numpy as np
+import pandas as pd
 import scipy.stats as stats
-from libpysal.weights.spatial_lag import lag_spatial as slag
+from libpysal.weights import W
+from libpysal.weights.spatial_lag import lag_spatial
+from matplotlib import colors
 from scipy import sparse
 
 from .crand import _prepare_univariate
@@ -34,7 +38,24 @@ __all__ = [
 PERMUTATIONS = 999
 
 
-class Moran(object):
+def _slag(w, y):
+    """Helper to compute lag either for W or for Graph"""
+    if isinstance(w, W):
+        return lag_spatial(w, y)
+    else:
+        return w.lag(y)
+
+
+def _transform(w, transformation):
+    """Helper to transform W or Graph"""
+    if isinstance(w, W):
+        w.transform = transformation
+        return w
+    else:
+        return w.transform(transformation)
+
+
+class Moran:
     """Moran's I Global Autocorrelation Statistic
 
     Parameters
@@ -42,8 +63,8 @@ class Moran(object):
 
     y               : array
                       variable measured across n spatial units
-    w               : W
-                      spatial weights instance
+    w               : W | Graph
+                      spatial weights instance as W or Graph aligned with y
     transformation  : string
                       weights transformation,  default is row-standardized "r".
                       Other options include "B": binary,  "D":
@@ -60,7 +81,7 @@ class Moran(object):
     ----------
     y            : array
                    original variable
-    w            : W
+    w            : W | Graph
                    original w object
     permutations : int
                    number of permutations
@@ -159,11 +180,11 @@ class Moran(object):
     ):
         y = np.asarray(y).flatten()
         self.y = y
-        w.transform = transformation
+        w = _transform(w, transformation)
         self.w = w
         self.permutations = permutations
         self.__moments()
-        self.I = self.__calc(self.z)  # noqa E741
+        self.I = self.__calc(self.z)  # noqa: E741
         self.z_norm = (self.I - self.EI) / self.seI_norm
         self.z_rand = (self.I - self.EI) / self.seI_rand
 
@@ -210,9 +231,15 @@ class Moran(object):
         self.EI = -1.0 / (self.n - 1)
         n = self.n
         n2 = n * n
-        s1 = self.w.s1
-        s0 = self.w.s0
-        s2 = self.w.s2
+        if isinstance(self.w, W):
+            s1 = self.w.s1
+            s0 = self.w.s0
+            s2 = self.w.s2
+        else:
+            self.summary = self.w.summary()
+            s1 = self.summary.s1
+            s0 = self.summary.s0
+            s2 = self.summary.s2
         s02 = s0 * s0
         v_num = n2 * s1 - n * s2 + 3 * s02
         v_den = (n - 1) * (n + 1) * s02
@@ -233,9 +260,10 @@ class Moran(object):
         self.seI_rand = VIR ** (1 / 2.0)
 
     def __calc(self, z):
-        zl = slag(self.w, z)
+        zl = _slag(self.w, z)
         inum = (z * zl).sum()
-        return self.n / self.w.s0 * inum / self.z2ss
+        s0 = self.w.s0 if isinstance(self.w, W) else self.summary.s0
+        return self.n / s0 * inum / self.z2ss
 
     @property
     def _statistic(self):
@@ -255,9 +283,9 @@ class Moran(object):
             a pandas dataframe with a geometry column
         cols : string or list of string
             name or list of names of columns to use to compute the statistic
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe inplace or to
             return a series contaning the results of the computation. If
@@ -288,11 +316,11 @@ class Moran(object):
             outvals=outvals,
             stat=cls,
             swapname=cls.__name__.lower(),
-            **stat_kws
+            **stat_kws,
         )
 
 
-class Moran_BV(object):
+class Moran_BV:  # noqa: N801
     """
     Bivariate Moran's I
 
@@ -302,8 +330,8 @@ class Moran_BV(object):
         x-axis variable
     y : array
         wy will be on y axis
-    w : W
-        weight instance assumed to be aligned with y
+    w : W | Graph
+        spatial weights instance as W or Graph aligned with x and y
     transformation  : {'R', 'B', 'D', 'U', 'V'}
                       weights transformation, default is row-standardized "r".
                       Other options include
@@ -321,7 +349,7 @@ class Moran_BV(object):
                     original x variable standardized by mean and std
     zy            : array
                     original y variable standardized by mean and std
-    w             : W
+    w             : W | Graph
                     original w object
     permutation   : int
                     number of permutations
@@ -380,7 +408,7 @@ class Moran_BV(object):
     >>> w = libpysal.io.open(libpysal.examples.get_path("sids2.gal")).read()
 
     Create an instance of Moran_BV
-    
+
     >>> from esda.moran import Moran_BV
     >>> mbi = Moran_BV(SIDR79,  SIDR74,  w)
 
@@ -408,9 +436,9 @@ class Moran_BV(object):
         self.zy = zy
         n = x.shape[0]
         self.den = n - 1.0  # zx'zx = zy'zy = n-1
-        w.transform = transformation
+        w = _transform(w, transformation)
         self.w = w
-        self.I = self.__calc(zy)  # noqa E741
+        self.I = self.__calc(zy)  # noqa: E741
         if permutations:
             nrp = np.random.permutation
             sim = [self.__calc(nrp(zy)) for i in range(permutations)]
@@ -430,7 +458,7 @@ class Moran_BV(object):
                 self.p_z_sim = stats.norm.cdf(self.z_sim)
 
     def __calc(self, zy):
-        wzy = slag(self.w, zy)
+        wzy = _slag(self.w, zy)
         self.num = (self.zx * wzy).sum()
         return self.num / self.den
 
@@ -449,7 +477,7 @@ class Moran_BV(object):
         inplace=False,
         pvalue="sim",
         outvals=None,
-        **stat_kws
+        **stat_kws,
     ):
         """
         Function to compute a Moran_BV statistic on a dataframe
@@ -466,9 +494,9 @@ class Moran_BV(object):
             column name or list of column names to use as Y values to compute
             the bivariate statistic. if no Y is provided, pariwise comparisons
             among the X variates are used instead.
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe inplace or to
             return a series contaning the results of the computation. If
@@ -501,11 +529,11 @@ class Moran_BV(object):
             outvals=outvals,
             swapname=cls.__name__.lower(),
             stat=cls,
-            **stat_kws
+            **stat_kws,
         )
 
 
-def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
+def Moran_BV_matrix(variables, w, permutations=0, varnames=None):  # noqa: N802
     """
     Bivariate Moran Matrix
 
@@ -515,8 +543,8 @@ def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
     ----------
     variables    : array or pandas.DataFrame
                    sequence of variables to be assessed
-    w            : W
-                   a spatial weights object
+    w            : W | Graph
+                   spatial weights instance as W or Graph aligned with variables
     permutations : int
                    number of permutations
     varnames     : list, optional if variables is an array
@@ -582,7 +610,7 @@ def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
     return results
 
 
-def _Moran_BV_Matrix_array(variables, w, permutations=0, varnames=None):
+def _Moran_BV_Matrix_array(variables, w, permutations=0, varnames=None):  # noqa: N802
     """
     Base calculation for MORAN_BV_Matrix
     """
@@ -604,7 +632,7 @@ def _Moran_BV_Matrix_array(variables, w, permutations=0, varnames=None):
     return results
 
 
-class Moran_Rate(Moran):
+class Moran_Rate(Moran):  # noqa: N801
     """
     Adjusted Moran's I Global Autocorrelation Statistic for Rate
     Variables :cite:`Assuncao1999`
@@ -617,8 +645,8 @@ class Moran_Rate(Moran):
     b               : array
                       a population-at-risk variable measured across n spatial
                       units
-    w               : W
-                      spatial weights instance
+    w               : W | Graph
+                      spatial weights instance as W or Graph aligned with e and b
     adjusted        : boolean
                       whether or not Moran's I needs to be adjusted for rate
                       variable
@@ -642,7 +670,7 @@ class Moran_Rate(Moran):
                    rate variable computed from parameters e and b
                    if adjusted is True, y is standardized rates
                    otherwise, y is raw rates
-    w            : W
+    w            : W | Graph
                    original w object
     permutations : int
                    number of permutations
@@ -722,10 +750,7 @@ class Moran_Rate(Moran):
     ):
         e = np.asarray(e).flatten()
         b = np.asarray(b).flatten()
-        if adjusted:
-            y = assuncao_rate(e, b)
-        else:
-            y = e * 1.0 / b
+        y = assuncao_rate(e, b) if adjusted else e * 1.0 / b
         Moran.__init__(
             self,
             y,
@@ -746,7 +771,7 @@ class Moran_Rate(Moran):
         pvalue="sim",
         outvals=None,
         swapname="",
-        **stat_kws
+        **stat_kws,
     ):
         """
         Function to compute a Moran_Rate statistic on a dataframe
@@ -763,9 +788,9 @@ class Moran_Rate(Moran):
             used for all event columns. If more than one population column
             is provided but there is not a population for every event
             column, an exception will be raised.
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe inplace or to
             return a series contaning the results of the computation. If
@@ -799,7 +824,7 @@ class Moran_Rate(Moran):
                 pvalue=pvalue,
                 outvals=outvals,
                 swapname=swapname,
-                **stat_kws
+                **stat_kws,
             )
             return new
         if isinstance(events, str):
@@ -822,12 +847,12 @@ class Moran_Rate(Moran):
 
         rates = [
             assuncao_rate(df[e], df[pop]) if adj else df[e].astype(float) / df[pop]
-            for e, pop, adj in zip(events, populations, adjusted)
+            for e, pop, adj in zip(events, populations, adjusted, strict=True)
         ]
-        names = ["-".join((e, p)) for e, p in zip(events, populations)]
+        names = ["-".join((e, p)) for e, p in zip(events, populations, strict=True)]
         out_df = df.copy()
         rate_df = out_df.from_dict(
-            dict(zip(names, rates))
+            dict(zip(names, rates, strict=True))
         )  # trick to avoid importing pandas
         stat_df = _univariate_handler(
             rate_df,
@@ -838,7 +863,7 @@ class Moran_Rate(Moran):
             outvals=outvals,
             swapname=swapname,
             stat=Moran,  # how would this get done w/super?
-            **stat_kws
+            **stat_kws,
         )
         for col in stat_df.columns:
             df[col] = stat_df[col]
@@ -849,7 +874,7 @@ class Moran_Rate(Moran):
 # -----------------------------------------------------------------------------#
 
 
-class Moran_Local(object):
+class Moran_Local:  # noqa: N801
     """Local Moran Statistics.
 
 
@@ -857,8 +882,8 @@ class Moran_Local(object):
     ----------
     y : array
         (n,1), attribute array
-    w : W
-        weight instance assumed to be aligned with y
+    w : W | Graph
+        spatial weights instance as W or Graph aligned with y
     transformation : {'R', 'B', 'D', 'U', 'V'}
          weights transformation,  default is row-standardized "r".
          Other options include
@@ -896,7 +921,7 @@ class Moran_Local(object):
 
     y : array
         original variable
-    w : W
+    w : W | Graph
         original w object
     permutations : int
         number of random permutations for calculation of pseudo p_values
@@ -1006,7 +1031,7 @@ class Moran_Local(object):
         n_jobs=1,
         keep_simulations=True,
         seed=None,
-        island_weight=0,
+        island_weight=0,  # noqa: ARG002
     ):
         y = np.asarray(y).flatten()
         self.y = y
@@ -1021,7 +1046,7 @@ class Moran_Local(object):
         z /= sy
         np.seterr(**orig_settings)
         self.z = z
-        w.transform = transformation
+        w = _transform(w, transformation)
         self.w = w
         self.permutations = permutations
         self.den = (z * z).sum()
@@ -1067,11 +1092,11 @@ class Moran_Local(object):
                 self.p_z_sim = np.nan
 
     def __calc(self, w, z):
-        zl = slag(w, z)
+        zl = _slag(w, z)
         return self.n_1 * self.z * zl / self.den
 
     def __quads(self):
-        zl = slag(self.w, self.z)
+        zl = _slag(self.w, self.z)
         zp = self.z > 0
         lp = zl > 0
         pp = zp * lp
@@ -1098,9 +1123,9 @@ class Moran_Local(object):
         # ---------------------------------------------------------
         expectation = -(z**2 * wi) / ((n - 1) * m2)
         var_term1 = (z / m2) ** 2
-        var_term2 = (n / (n - 2))
-        var_term3 = (wi2 - (wi**2 / (n - 1)))
-        var_term4 = (m2 - (z**2 / (n - 1)))
+        var_term2 = n / (n - 2)
+        var_term3 = wi2 - (wi**2 / (n - 1))
+        var_term4 = m2 - (z**2 / (n - 1))
         variance = var_term1 * var_term2 * var_term3 * var_term4
 
         self.EIc = expectation
@@ -1146,9 +1171,9 @@ class Moran_Local(object):
             a pandas dataframe with a geometry column
         cols : string or list of string
             name or list of names of columns to use to compute the statistic
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe inplace or to
             return a series contaning the results of the computation. If
@@ -1180,11 +1205,48 @@ class Moran_Local(object):
             outvals=outvals,
             stat=cls,
             swapname=cls.__name__.lower(),
-            **stat_kws
+            **stat_kws,
         )
 
+    def get_cluster_labels(self, crit_value=0.05):
+        """Return LISA cluster labels for each observation.
 
-class Moran_Local_BV(object):
+        Parameters
+        ----------
+        crit_value : float, optional
+            crititical significance value for statistical inference, by default 0.05
+
+        Returns
+        -------
+        numpy.array
+            an array of cluster labels aligned with the input data used to conduct the
+            local Moran analysis
+        """
+        return _get_cluster_labels(self, crit_value)
+
+    def explore(self, gdf, crit_value=0.05, **kwargs):
+        """Create interactive map of LISA indicators
+
+        Parameters
+        ----------
+        gdf : geopandas.GeoDataFrame
+            geodataframe used to conduct the local Moran analysis
+        crit_value : float, optional
+            critical value to determine statistical significance, by default 0.05
+        kwargs : dict, optional
+            additional keyword arguments passed to the geopandas `explore` method
+
+        Returns
+        -------
+        Folium.Map
+            interactive map with LISA clusters
+        """
+        gdf = gdf.copy()
+        gdf["Moran Cluster"] = self.get_cluster_labels(crit_value)
+        return _explore_local_moran(self, gdf, crit_value, **kwargs)
+
+
+class Moran_Local_BV:  # noqa: N801
     """Bivariate Local Moran Statistics.
 
 
@@ -1194,8 +1256,8 @@ class Moran_Local_BV(object):
         x-axis variable
     y : array
         (n,1), wy will be on y axis
-    w : W
-        weight instance assumed to be aligned with y
+    w : W | Graph
+        spatial weights instance as W or Graph aligned with y
     transformation : {'R', 'B', 'D', 'U', 'V'}
         weights transformation,  default is row-standardized "r".
         Other options include
@@ -1234,7 +1296,7 @@ class Moran_Local_BV(object):
         original x variable standardized by mean and std
     zy : array
         original y variable standardized by mean and std
-    w : W
+    w : W | Graph
         original w object
     permutations : int
         number of random permutations for calculation of pseudo p_values
@@ -1305,7 +1367,7 @@ class Moran_Local_BV(object):
         n_jobs=1,
         keep_simulations=True,
         seed=None,
-        island_weight=0,
+        island_weight=0,  # noqa: ARG002
     ):
         x = np.asarray(x).flatten()
         y = np.asarray(y).flatten()
@@ -1327,11 +1389,11 @@ class Moran_Local_BV(object):
         np.seterr(**orig_settings)
         self.zx = zx
         self.zy = zy
-        w.transform = transformation
+        w = _transform(w, transformation)
         self.w = w
         self.permutations = permutations
         self.den = (zx * zx).sum()
-        self.Is = self.__calc(self.w, self.zx, self.zy)
+        self.Is = self.__calc()
         self.geoda_quads = geoda_quads
         quads = [1, 2, 3, 4]
         if geoda_quads:
@@ -1364,12 +1426,12 @@ class Moran_Local_BV(object):
                 self.z_sim = (self.Is - self.EI_sim) / self.seI_sim
                 self.p_z_sim = stats.norm.sf(np.abs(self.z_sim))
 
-    def __calc(self, w, zx, zy):
-        zly = slag(w, zy)
+    def __calc(self):
+        zly = _slag(self.w, self.zy)
         return self.n_1 * self.zx * zly / self.den
 
     def __quads(self):
-        zl = slag(self.w, self.zy)
+        zl = _slag(self.w, self.zy)
         zp = self.zx > 0
         lp = zl > 0
         pp = zp * lp
@@ -1395,7 +1457,7 @@ class Moran_Local_BV(object):
         inplace=False,
         pvalue="sim",
         outvals=None,
-        **stat_kws
+        **stat_kws,
     ):
         """
         Function to compute a Moran_Local_BV statistic on a dataframe
@@ -1412,9 +1474,9 @@ class Moran_Local_BV(object):
             column name or list of column names to use as Y values to compute
             the bivariate statistic. if no Y is provided, pariwise comparisons
             among the X variates are used instead.
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe inplace or to
             return a series contaning the results of the computation. If
@@ -1447,11 +1509,11 @@ class Moran_Local_BV(object):
             outvals=outvals,
             swapname=cls.__name__.lower(),
             stat=cls,
-            **stat_kws
+            **stat_kws,
         )
 
 
-class Moran_Local_Rate(Moran_Local):
+class Moran_Local_Rate(Moran_Local):  # noqa: N801
     """
     Adjusted Local Moran Statistics for Rate Variables :cite:`Assuncao1999`.
 
@@ -1461,8 +1523,8 @@ class Moran_Local_Rate(Moran_Local):
         (n,1), an event variable across n spatial units
     b : array
         (n,1), a population-at-risk variable across n spatial units
-    w : W
-        weight instance assumed to be aligned with y
+    w : W | Graph
+        spatial weights instance as W or Graph aligned with y
     adjusted : boolean
         whether or not local Moran statistics need to be adjusted for
         rate variable
@@ -1502,7 +1564,7 @@ class Moran_Local_Rate(Moran_Local):
         rate variables computed from parameters e and b
         if adjusted is True, y is standardized rates
         otherwise, y is raw rates
-    w : W
+    w : W | Graph
         original w object
     permutations : int
         number of random permutations for calculation of pseudo
@@ -1577,14 +1639,11 @@ class Moran_Local_Rate(Moran_Local):
         n_jobs=1,
         keep_simulations=True,
         seed=None,
-        island_weight=0,
+        island_weight=0,  # noqa: ARG002
     ):
         e = np.asarray(e).flatten()
         b = np.asarray(b).flatten()
-        if adjusted:
-            y = assuncao_rate(e, b)
-        else:
-            y = e * 1.0 / b
+        y = assuncao_rate(e, b) if adjusted else e * 1.0 / b
         Moran_Local.__init__(
             self,
             y,
@@ -1608,7 +1667,7 @@ class Moran_Local_Rate(Moran_Local):
         pvalue="sim",
         outvals=None,
         swapname="",
-        **stat_kws
+        **stat_kws,
     ):
         """
         Function to compute a Moran_Local_Rate statistic on a dataframe
@@ -1625,9 +1684,9 @@ class Moran_Local_Rate(Moran_Local):
             used for all event columns. If more than one population column
             is provided but there is not a population for every event
             column, an exception will be raised.
-        w : pysal weights object
-            a weights object aligned with the dataframe. If not provided, this
-            is searched for in the dataframe's metadata
+        w : W | Graph
+            spatial weights instance as W or Graph aligned with the dataframe. If not
+            provided, this is searched for in the dataframe's metadata
         inplace : bool
             a boolean denoting whether to operate on the dataframe
             inplace or to return a series contaning the results of
@@ -1661,7 +1720,7 @@ class Moran_Local_Rate(Moran_Local):
                 pvalue=pvalue,
                 outvals=outvals,
                 swapname=swapname,
-                **stat_kws
+                **stat_kws,
             )
             return new
         if isinstance(events, str):
@@ -1684,12 +1743,12 @@ class Moran_Local_Rate(Moran_Local):
 
         rates = [
             assuncao_rate(df[e], df[pop]) if adj else df[e].astype(float) / df[pop]
-            for e, pop, adj in zip(events, populations, adjusted)
+            for e, pop, adj in zip(events, populations, adjusted, strict=True)
         ]
-        names = ["-".join((e, p)) for e, p in zip(events, populations)]
+        names = ["-".join((e, p)) for e, p in zip(events, populations, strict=True)]
         out_df = df.copy()
         rate_df = out_df.from_dict(
-            dict(zip(names, rates))
+            dict(zip(names, rates, strict=True))
         )  # trick to avoid importing pandas
         _univariate_handler(
             rate_df,
@@ -1700,12 +1759,71 @@ class Moran_Local_Rate(Moran_Local):
             outvals=outvals,
             swapname=swapname,
             stat=Moran_Local,  # how would this get done w/super?
-            **stat_kws
+            **stat_kws,
         )
         for col in rate_df.columns:
             df[col] = rate_df[col]
 
 
+def _explore_local_moran(moran_local, gdf, crit_value, **kwargs):
+    """Plot local Moran values as an interactive map
+
+    Parameters
+    ----------
+    moran_local : esda.Moran_Local
+        a fitted local Moran class from the PySAL esda module
+    gdf : geopandas.GeoDataFrame
+        geodataframe used to create the Moran_Local class
+    crit_value : float, optional
+        critical value for determining statistical significance, by default 0.05
+    kwargs : dict, optional
+        additional keyword arguments are passed directly
+        to geopandas.explore, by default None
+
+    Returns
+    -------
+    m
+        folium.Map
+    """
+
+    gdf = gdf.copy()
+    gdf["Moran Cluster"] = moran_local.get_cluster_labels(crit_value)
+    gdf["p-value"] = moran_local.p_sim
+
+    x = gdf["Moran Cluster"].values
+    y = np.unique(x)
+    colors5_mpl = {
+        "High-High": "#d7191c",
+        "Low-High": "#89cff0",
+        "Low-Low": "#2c7bb6",
+        "High-Low": "#fdae61",
+        "Insignificant": "lightgrey",
+    }
+    colors5 = [colors5_mpl[i] for i in y]  # for mpl
+    hmap = colors.ListedColormap(colors5)
+    if "cmap" not in kwargs:
+        kwargs["cmap"] = hmap
+
+    m = gdf[["Moran Cluster", "p-value", "geometry"]].explore("Moran Cluster", **kwargs)
+    return m
+
+
+def _get_cluster_labels(moran_local, crit_value):
+    gdf = pd.DataFrame()
+    gdf["q"] = moran_local.q
+    gdf["p_sim"] = moran_local.p_sim
+    gdf["Moran Cluster"] = "Insignificant"
+
+    gdf.loc[(gdf["p_sim"] < crit_value) & (gdf["q"] == 1), "Moran Cluster"] = (
+        "High-High"
+    )
+    gdf.loc[(gdf["p_sim"] < crit_value) & (gdf["q"] == 2), "Moran Cluster"] = "Low-High"
+    gdf.loc[(gdf["p_sim"] < crit_value) & (gdf["q"] == 3), "Moran Cluster"] = "Low-Low"
+    gdf.loc[(gdf["p_sim"] < crit_value) & (gdf["q"] == 4), "Moran Cluster"] = "High-Low"
+
+    return gdf["Moran Cluster"].values
+
+
 # --------------------------------------------------------------
 # Conditional Randomization Moment Estimators
 # --------------------------------------------------------------
@@ -1794,10 +1912,9 @@ def _wikh_slow(W, sokal_correction=False):
                 # excluding wik * wii
                 if i == h:
                     continue
-                if sokal_correction:
+                if sokal_correction and h == k:
                     # excluding wih * wih
-                    if h == k:
-                        continue
+                    continue
                 acc += W[i, k] * W[i, h]
         result[i] = acc
     return result / 2
