@@ -1,35 +1,47 @@
+import contextlib
+
 import numpy
 import pandas
+from packaging.version import Version
+
+# gets handled at the _cast level.
+with contextlib.suppress(ImportError, ModuleNotFoundError):
+    import shapely
 
-try:
-    import pygeos
-except (ImportError, ModuleNotFoundError):
-    pass  # gets handled at the _cast level.
 
 from .crand import njit, prange
 
+__author__ = (
+    "Martin Fleischmann <martin@fleischmann.net>",
+    "Levi John Wolf <levi.john.wolf@gmail.com>",
+    "Alan Murray <amurray@ucsb.edu>",
+    "Jiwan Baik <jiwon.baik@geog.ucsb.edu>",
+)
+
 
 # -------------------- UTILITIES --------------------#
 def _cast(collection):
     """
-    Cast a collection to a pygeos geometry array.
+    Cast a collection to a shapely geometry array.
     """
     try:
-        import pygeos, geopandas
+        import geopandas
+        import shapely
     except (ImportError, ModuleNotFoundError) as exception:
-        raise type(exception)("pygeos and geopandas are required for shape statistics.")
+        raise type(exception)(
+            "shapely and geopandas are required for shape statistics."
+        ) from None
 
-    if isinstance(collection, (geopandas.GeoSeries, geopandas.GeoDataFrame)):
-        return collection.geometry.values.data.squeeze()
-    elif pygeos.is_geometry(collection).all():
-        if isinstance(collection, (numpy.ndarray, list)):
+    if Version(shapely.__version__) < Version("2"):
+        raise ImportError("Shapely 2.0 or newer is required.")
+
+    if isinstance(collection, geopandas.GeoSeries | geopandas.GeoDataFrame):
+        return numpy.asarray(collection.geometry.array)
+    else:
+        if isinstance(collection, numpy.ndarray | list):
             return numpy.asarray(collection)
         else:
             return numpy.array([collection])
-    elif isinstance(collection, (numpy.ndarray, list)):
-        return pygeos.from_shapely(collection).squeeze()
-    else:
-        return numpy.array([pygeos.from_shapely(collection)])
 
 
 def get_angles(collection, return_indices=False):
@@ -39,7 +51,7 @@ def get_angles(collection, return_indices=False):
 
     Parameters
     ----------
-    ga  :   pygeos geometry array
+    ga  :   shapely geometry array
         array of polygons/multipolygons
     return_indices  :   bool (Default: False)
         whether to return the indices relating each geometry to a polygon
@@ -55,23 +67,22 @@ def get_angles(collection, return_indices=False):
     -------
     If a geometry has n coordinates and k parts, the array will be n - k.
     If each geometry has n_i coordinates, then let N be a vector storing
-    those counts (computed, for example, using pygeos.get_num_coordinates(ga)).
+    those counts (computed, for example, using shapely.get_num_coordinates(ga)).
     Likewise, let K be a vector storing the number of parts each geometry has, k_i
-    (computed, for example, using pygeos.get_num_geometries(ga))
+    (computed, for example, using shapely.get_num_geometries(ga))
 
     Then, the output is of shape (N - K).sum()
 
     """
     ga = _cast(collection)
-    exploded = pygeos.get_parts(ga)
-    coords = pygeos.get_coordinates(exploded)
-    n_coords_per_geom = pygeos.get_num_coordinates(exploded)
-    n_parts_per_geom = pygeos.get_num_geometries(exploded)
+    exploded = shapely.get_parts(ga)
+    coords = shapely.get_coordinates(exploded)
+    n_coords_per_geom = shapely.get_num_coordinates(exploded)
     angles = numpy.asarray(_get_angles(coords, n_coords_per_geom))
     if return_indices:
         return angles, numpy.repeat(
             numpy.arange(len(ga)),
-            pygeos.get_num_coordinates(ga) - pygeos.get_num_geometries(ga),
+            shapely.get_num_coordinates(ga) - shapely.get_num_geometries(ga),
         )
     else:
         return angles
@@ -84,12 +95,10 @@ def _get_angles(points, n_coords_per_geom):
     This assumes that the input geometries are simple, not multi!
     """
     # Start at the first point of the first geometry
-    offset = int(0)
-    start = points[0]
+    offset = 0
     on_geom = 0
     on_coord = 0
     result = []
-    n_points = len(points)
     while True:
         # if we're on the last point before the closure point,
         if on_coord == (n_coords_per_geom[on_geom] - 1):
@@ -115,7 +124,7 @@ def _get_angles(points, n_coords_per_geom):
         a = left - center
         b = right - center
         # compute the angle between the segments
-        angle = numpy.math.atan2(a[0] * b[1] - a[1] * b[0], numpy.dot(a, b))
+        angle = numpy.arctan2(a[0] * b[1] - a[1] * b[0], numpy.dot(a, b))
         result.append(angle)
         on_coord += 1
     return result
@@ -125,11 +134,11 @@ def _get_angles(points, n_coords_per_geom):
 
 
 def isoperimetric_quotient(collection):
-    """
+    r"""
     The Isoperimetric quotient, defined as the ratio of a polygon's area to the
     area of the equi-perimeter circle.
 
-    Altman's PA_1 measure
+    Altman's PA_1 measure :cite:`altman1998Districting`
 
     Construction:
 
@@ -154,7 +163,7 @@ def isoperimetric_quotient(collection):
     pp = (a_d) / (a_c) = (a_d) / ((p_d / (2*\pi))^2 * \pi) = (a_d) / (p_d**2 / (4\PI))
     """
     ga = _cast(collection)
-    return (4 * numpy.pi * pygeos.area(ga)) / (pygeos.measurement.length(ga) ** 2)
+    return (4 * numpy.pi * shapely.area(ga)) / (shapely.measurement.length(ga) ** 2)
 
 
 def isoareal_quotient(collection):
@@ -163,11 +172,12 @@ def isoareal_quotient(collection):
     perimeter of the equi-areal circle
 
     Altman's PA_3 measure, and proportional to the PA_4 measure
+    :cite:`altman1998Districting`
     """
     ga = _cast(collection)
     return (
-        2 * numpy.pi * numpy.sqrt(pygeos.area(ga) / numpy.pi)
-    ) / pygeos.measurement.length(ga)
+        2 * numpy.pi * numpy.sqrt(shapely.area(ga) / numpy.pi)
+    ) / shapely.measurement.length(ga)
 
 
 def minimum_bounding_circle_ratio(collection):
@@ -175,42 +185,43 @@ def minimum_bounding_circle_ratio(collection):
     The Reock compactness measure, defined by the ratio of areas between the
     minimum bounding/containing circle of a shape and the shape itself.
 
-    Measure A1 in Altman (1998), cited for Frolov (1974), but earlier from Reock
+    Measure A1 in :cite:`altman1998Districting`,
+    cited for Frolov (1974), but earlier from Reock
     (1963)
     """
     ga = _cast(collection)
-    mbc = pygeos.minimum_bounding_circle(ga)
-    return pygeos.area(ga) / pygeos.area(mbc)
+    mbca = (shapely.minimum_bounding_radius(ga) ** 2) * numpy.pi
+    return shapely.area(ga) / mbca
 
 
 def radii_ratio(collection):
     """
-    The Flaherty & Crumplin (1992) index, OS_3 in Altman (1998).
+    The Flaherty & Crumplin (1992) index, OS_3 in :cite:`altman1998Districting`.
 
     The ratio of the radius of the equi-areal circle to the radius of the MBC
     """
     ga = _cast(collection)
-    r_eac = numpy.sqrt(pygeos.area(ga) / numpy.pi)
-    r_mbc = pygeos.minimum_bounding_radius(ga)
+    r_eac = numpy.sqrt(shapely.area(ga) / numpy.pi)
+    r_mbc = shapely.minimum_bounding_radius(ga)
     return r_eac / r_mbc
 
 
 def diameter_ratio(collection, rotated=True):
     """
     The Flaherty & Crumplin (1992) length-width measure, stated as measure LW_7
-    in Altman (1998).
+    in :cite:`altman1998Districting`.
 
     It is given as the ratio between the minimum and maximum shape diameter.
     """
     ga = _cast(collection)
     if rotated:
-        box = pygeos.minimum_rotated_rectangle(ga)
-        coords = pygeos.get_coordinates(box)
-        a, b, c, d = (coords[0::5], coords[1::5], coords[2::5], coords[3::5])
+        box = shapely.minimum_rotated_rectangle(ga)
+        coords = shapely.get_coordinates(box)
+        a, b, _, d = (coords[0::5], coords[1::5], coords[2::5], coords[3::5])
         widths = numpy.sqrt(numpy.sum((a - b) ** 2, axis=1))
         heights = numpy.sqrt(numpy.sum((a - d) ** 2, axis=1))
     else:
-        box = pygeos.bounds(ga)
+        box = shapely.bounds(ga)
         (xmin, xmax), (ymin, ymax) = box[:, [0, 2]].T, box[:, [1, 3]].T
         widths, heights = numpy.abs(xmax - xmin), numpy.abs(ymax - ymin)
     return numpy.minimum(widths, heights) / numpy.maximum(widths, heights)
@@ -225,10 +236,10 @@ def length_width_diff(collection):
     Where L is the maximal east-west extent and W is the maximal north-south
     extent.
 
-    Defined as measure LW_5 in Altman (1998)
+    Defined as measure LW_5 in :cite:`altman1998Districting`
     """
     ga = _cast(collection)
-    box = pygeos.bounds(ga)
+    box = shapely.bounds(ga)
     (xmin, xmax), (ymin, ymax) = box[:, [0, 2]].T, box[:, [1, 3]].T
     width, height = numpy.abs(xmax - xmin), numpy.abs(ymax - ymin)
     return width - height
@@ -247,9 +258,7 @@ def boundary_amplitude(collection):
     between zero and one, like many of the other ideal shape-based indices.
     """
     ga = _cast(collection)
-    return pygeos.measurement.length(
-        pygeos.convex_hull(ga)
-    ) / pygeos.measurement.length(ga)
+    return shapely.length(shapely.convex_hull(ga)) / shapely.length(ga)
 
 
 def convex_hull_ratio(collection):
@@ -259,18 +268,21 @@ def convex_hull_ratio(collection):
     Altman's A_3 measure, from Neimi et al 1991.
     """
     ga = _cast(collection)
-    return pygeos.area(ga) / pygeos.area(pygeos.convex_hull(ga))
+    return shapely.area(ga) / shapely.area(shapely.convex_hull(ga))
 
 
 def fractal_dimension(collection, support="hex"):
     """
-    The fractal dimension of the boundary of a shape, assuming a given spatial support for the geometries.
+    The fractal dimension of the boundary of a shape, assuming a given
+    spatial support for the geometries.
 
-    Note that this derivation assumes a specific ideal spatial support for the polygon, and is thus may not return valid results for complex or highly irregular geometries.
+    Note that this derivation assumes a specific ideal spatial support
+    for the polygon, and is thus may not return valid results for
+    complex or highly irregular geometries.
     """
     ga = _cast(collection)
-    P = pygeos.measurement.length(ga)
-    A = pygeos.area(ga)
+    P = shapely.length(ga)
+    A = shapely.area(ga)
     if support == "hex":
         return 2 * numpy.log(P / 6) / numpy.log(A / (3 * numpy.sin(numpy.pi / 3)))
     elif support == "square":
@@ -279,7 +291,8 @@ def fractal_dimension(collection, support="hex"):
         return 2 * numpy.log(P / (2 * numpy.pi)) / numpy.log(A / numpy.pi)
     else:
         raise ValueError(
-            f"The support argument must be one of 'hex', 'circle', or 'square', but {support} was provided."
+            "The support argument must be one of 'hex', 'circle', or 'square', "
+            f"but {support} was provided."
         )
 
 
@@ -310,19 +323,21 @@ def squareness(collection):
 
     """
     ga = _cast(collection)
-    return ((numpy.sqrt(pygeos.area(ga)) * 4) / pygeos.length(ga)) ** 2
+    return ((numpy.sqrt(shapely.area(ga)) * 4) / shapely.length(ga)) ** 2
 
 
 def rectangularity(collection):
     """
-    Ratio of the area of the shape to the area of its minimum bounding rotated rectangle
+    Ratio of the area of the shape to the area
+    of its minimum bounding rotated rectangle
 
     Reveals a polygon’s degree of being curved inward.
 
     .. math::
         \\frac{A}{A_{MBR}}
 
-    where :math:`A` is the area and :math:`A_{MBR}` is the area of minimum bounding
+    where :math:`A` is the area and :math:`A_{MBR}`
+    is the area of minimum bounding
     rotated rectangle.
 
     Notes
@@ -330,7 +345,7 @@ def rectangularity(collection):
     Implementation follows :cite:`basaraner2017`.
     """
     ga = _cast(collection)
-    return pygeos.area(ga) / pygeos.area(pygeos.minimum_rotated_rectangle(ga))
+    return shapely.area(ga) / shapely.area(shapely.minimum_rotated_rectangle(ga))
 
 
 def shape_index(collection):
@@ -349,7 +364,7 @@ def shape_index(collection):
 
     """
     ga = _cast(collection)
-    return numpy.sqrt(pygeos.area(ga) / numpy.pi) / pygeos.minimum_bounding_radius(ga)
+    return numpy.sqrt(shapely.area(ga) / numpy.pi) / shapely.minimum_bounding_radius(ga)
 
 
 def equivalent_rectangular_index(collection):
@@ -370,9 +385,9 @@ def equivalent_rectangular_index(collection):
     Implementation follows :cite:`basaraner2017`.
     """
     ga = _cast(collection)
-    box = pygeos.minimum_rotated_rectangle(ga)
-    return numpy.sqrt(pygeos.area(ga) / pygeos.area(box)) * (
-        pygeos.length(box) / pygeos.length(ga)
+    box = shapely.minimum_rotated_rectangle(ga)
+    return numpy.sqrt(shapely.area(ga) / shapely.area(box)) * (
+        shapely.length(box) / shapely.length(ga)
     )
 
 
@@ -394,7 +409,7 @@ def form_factor(collection, height):
     Implementation follows :cite:`bourdic2012`.
     """
     ga = _cast(collection)
-    A = pygeos.area(ga)
+    A = shapely.area(ga)
     V = A * height
     zeros = V == 0
     res = numpy.zeros(len(ga))
@@ -414,24 +429,25 @@ def moment_of_inertia(collection):
     Thus, for constant unit mass at each boundary point,
     the MoI of this pointcloud is
 
-    \sum_i d_{i,c}^2
+    .. math::
+        \\sum_i d_{i,c}^2
 
     where c is the centroid of the polygon
 
-    Altman's OS_1 measure, cited in Boyce and Clark (1964), also used in Weaver
-    and Hess (1963).
+    Altman's OS_1 measure :cite:`altman1998Districting`, cited in Boyce and Clark
+    (1964), also used in Weaver and Hess (1963).
     """
     ga = _cast(collection)
-    coords = pygeos.get_coordinates(ga)
-    geom_ixs = numpy.repeat(numpy.arange(len(ga)), pygeos.get_num_coordinates(ga))
-    centroids = pygeos.get_coordinates(pygeos.centroid(ga))[geom_ixs]
+    coords = shapely.get_coordinates(ga)
+    geom_ixs = numpy.repeat(numpy.arange(len(ga)), shapely.get_num_coordinates(ga))
+    centroids = shapely.get_coordinates(shapely.centroid(ga))[geom_ixs]
     squared_euclidean = numpy.sum((coords - centroids) ** 2, axis=1)
     dists = (
         pandas.DataFrame.from_dict(dict(d2=squared_euclidean, geom_ix=geom_ixs))
         .groupby("geom_ix")
         .d2.sum()
     ).values
-    return pygeos.area(ga) / numpy.sqrt(2 * dists)
+    return shapely.area(ga) / numpy.sqrt(2 * dists)
 
 
 def moa_ratio(collection):
@@ -440,8 +456,8 @@ def moa_ratio(collection):
     the moment of area of a circle with the same area.
     """
     ga = _cast(collection)
-    r = pygeos.measurement.length(ga) / (2 * numpy.pi)
-    return (numpy.pi * 0.5 * r ** 4) / second_areal_moment(ga)
+    r = shapely.length(ga) / (2 * numpy.pi)
+    return (numpy.pi * 0.5 * r**4) / second_areal_moment(ga)
 
 
 def nmi(collection):
@@ -451,48 +467,124 @@ def nmi(collection):
     its second moment of area.
     """
     ga = _cast(collection)
-    return pygeos.area(ga) ** 2 / (2 * second_areal_moment(ga) * numpy.pi)
+    return shapely.area(ga) ** 2 / (2 * second_areal_moment(ga) * numpy.pi)
 
 
 def second_areal_moment(collection):
     """
-    Using equation listed on en.wikipedia.org/Second_Moment_of_area, the second
-    moment of area is actually the cross-moment of area between the X and Y
-    dimensions:
+    Using equation listed on en.wikipedia.org/wiki/Second_moment_of_area#Any_polygon, the second
+    moment of area is the sum of the inertia across the x and y axes:
 
-    I_xy = (1/24)\sum^{i=N}^{i=1} (x_iy_{i+1} + 2*x_iy_i + 2*x_{i+1}y_{i+1} +
-    x_{i+1}y_i)(x_iy_i - x_{i+1}y_i)
+    The :math:`x` axis is given by:
 
-    where x_i, y_i is the current point and x_{i+1}, y_{i+1} is the next point,
-    and where x_{n+1} = x_1, y_{n+1} = 1.
+    .. math::
 
-    This relation is known as the:
-    - second moment of area
-    - moment of inertia of plane area
-    - area moment of inertia
-    - second area moment
+        I_x = (1/12)\\sum^{N}_{i=1} (x_i y_{i+1} - x_{i+1}y_i) (x_i^2 + x_ix_{i+1} + x_{i+1}^2)
 
-    and is *not* the mass moment of inertia, a property of the distribution of
-    mass around a shape.
-    """
+    While the :math:`y` axis is in a similar form:
+
+    .. math::
+
+        I_y = (1/12)\\sum^{N}_{i=1} (x_i y_{i+1} - x_{i+1}y_i) (y_i^2 + y_iy_{i+1} + y_{i+1}^2)
+
+    where :math:`x_i`, :math:`y_i` is the current point and :math:`x_{i+1}`, :math:`y_{i+1}` is the next point,
+    and where :math:`x_{n+1} = x_1, y_{n+1} = y_1`. For multipart polygons with holes,
+    all parts are treated as separate contributions to the overall centroid, which
+    provides the same result as if all parts with holes are separately computed, and then
+    merged together using the parallel axis theorem.
+
+    References
+    ----------
+    Hally, D. 1987. The calculations of the moments of polygons. Canadian National
+    Defense Research and Development Technical Memorandum 87/209.
+    https://apps.dtic.mil/dtic/tr/fulltext/u2/a183444.pdf
+
+    """  # noqa: E501
     ga = _cast(collection)
-    result = numpy.zeros(len(ga))
-    n_holes_per_geom = pygeos.get_num_interior_rings(ga)
-    for i, geometry in enumerate(ga):
-        n_holes = n_holes_per_geom[i]
-        for hole_ix in range(n_holes):
-            hole = pygeos.get_coordinates(pygeos.get_interior_ring(ga, hole_ix))
-            result[i] -= _second_moa_ring(hole)
-        n_parts = pygeos.get_num_geometries(geometry)
-        for part in pygeos.get_parts(geometry):
-            result[i] += _second_moa_ring(pygeos.get_coordinates(part))
-    # must divide everything by 24 and flip if polygon is clockwise.
-    signflip = numpy.array([-1, 1])[pygeos.is_ccw(ga).astype(int)]
-    return result * (1 / 24) * signflip
+    import geopandas  # function level, to follow module design
+
+    # construct a dataframe of the fundamental parts of all input polygons
+    parts, collection_ix = shapely.get_parts(ga, return_index=True)
+    rings, ring_ix = shapely.get_rings(parts, return_index=True)
+    # get_rings() always returns the exterior first, then the interiors
+    collection_ix = numpy.repeat(
+        collection_ix, shapely.get_num_interior_rings(parts) + 1
+    )
+    # we need to work in polygon-space for the algorithms
+    # (centroid, shoelace calculation) to work
+    polygon_rings = shapely.polygons(rings)
+    is_external = numpy.zeros_like(collection_ix).astype(bool)
+    # the first element is always external
+    is_external[0] = True
+    # and each subsequent element is external iff
+    # it is different from the preceeding index
+    is_external[1:] = ring_ix[1:] != ring_ix[:-1]
+    # now, our analysis frame contains a bunch of (guaranteed-to-be-simple) polygons
+    # that represent either exterior rings or holes
+    polygon_rings = geopandas.GeoDataFrame(
+        dict(
+            collection_ix=collection_ix,
+            ring_within_geom_ix=ring_ix,
+            is_external_ring=is_external,
+        ),
+        geometry=polygon_rings,
+    )
+    # the polygonal moi can be calculated using the same ring-based strategy,
+    # and this could be parallelized if necessary over the elemental shapes with:
+
+    # from joblib import Parallel, parallel_backend, delayed
+    # with parallel_backend('loky', n_jobs=-1):
+    #     engine = Parallel()
+    #     promise = delayed(_second_moment_of_area_polygon)
+    #     result = engine(promise(geom) for geom in polygon_rings.geometry.values)
+
+    # but we will keep simple for now
+    polygon_rings["moa"] = polygon_rings.geometry.apply(_second_moment_of_area_polygon)
+    # the above algorithm computes an unsigned moa
+    # to be insensitive to winding direction.
+    # however, we need to subtract the moa of holes. Hence, the sign of the moa is
+    # -1 when the polygon is an internal ring and 1 otherwise:
+    polygon_rings["sign"] = (1 - polygon_rings.is_external_ring * 2) * -1
+    # shapely already uses the correct formulation for centroids
+    polygon_rings["centroids"] = shapely.centroid(polygon_rings.geometry)
+    # the inertia of parts applies to the overall center of mass:
+    original_centroids = shapely.centroid(ga)
+    polygon_rings["collection_centroid"] = original_centroids[collection_ix]
+    # proportional to the squared distance between the original and part centroids:
+    polygon_rings["radius"] = shapely.distance(
+        polygon_rings.centroid.values, polygon_rings.collection_centroid.values
+    )
+    # now, we take the sum of (I+Ar^2) for each ring, treating the
+    # contribution of holes as negative.
+    # Then, we take the sum of all of the contributions
+    return (
+        polygon_rings.groupby(["collection_ix", "ring_within_geom_ix"])
+        .apply(
+            lambda ring_in_part: (
+                (ring_in_part.moa + ring_in_part.radius**2 * ring_in_part.area)
+                * ring_in_part.sign
+            ).sum()
+        )
+        .groupby(level="collection_ix")
+        .sum()
+        .values
+    )
+
+
+def _second_moment_of_area_polygon(polygon):
+    """
+    Compute the absolute value of the moment of area (i.e. ignoring winding direction)
+    for an input polygon.
+    """
+    coordinates = shapely.get_coordinates(polygon)
+    centroid = shapely.centroid(polygon)
+    centroid_coords = shapely.get_coordinates(centroid)
+    moi = _second_moa_ring_xplusy(coordinates - centroid_coords)
+    return abs(moi)
 
 
 @njit
-def _second_moa_ring(points):
+def _second_moa_ring_xplusy(points):
     """
     implementation of the moment of area for a single ring
     """
@@ -500,13 +592,17 @@ def _second_moa_ring(points):
     for i in prange(len(points[:-1])):
         x_tail, y_tail = points[i]
         x_head, y_head = points[i + 1]
-        moi += (x_tail * y_head - x_head * y_tail) * (
-            x_tail * y_head
-            + 2 * x_tail * y_tail
-            + 2 * x_head * y_head
-            + x_head * y_tail
+        xtyh = x_tail * y_head
+        xhyt = x_head * y_tail
+        moi += (xtyh - xhyt) * (
+            x_head**2
+            + x_head * x_tail
+            + x_tail**2
+            + y_head**2
+            + y_head * y_tail
+            + y_tail**2
         )
-    return moi
+    return moi / 12
 
 
 # -------------------- OTHER MEASURES -------------------- #
@@ -514,7 +610,7 @@ def _second_moa_ring(points):
 
 def reflexive_angle_ratio(collection):
     """
-    The Taylor reflexive angle index, measure OS_4 in Altman (1998)
+    The Taylor reflexive angle index, measure OS_4 in :cite:`altman1998Districting`
 
     (N-R)/(N+R), the difference in number between non-reflexive angles and
     reflexive angles in a polygon, divided by the number of angles in the
